Package: plot3D
Version: 1.4.2
Title: Plotting Multi-Dimensional Data
Authors@R: person(given = "Karline",
                    family = "Soetaert",
                    role = c("aut", "cre"),
                    email = "karline.soetaert@nioz.nl",
         					 comment = c(ORCID = "0000-0003-4603-7100"))
Depends: R (>= 2.15)
Imports: misc3d, stats, graphics, grDevices, methods
Description: Functions for viewing 2-D and 3-D data, including perspective plots, slice plots, surface plots, scatter plots, etc. Includes data sets from oceanography.
License: GPL (>= 3.0)
LazyData: yes
NeedsCompilation: no
Packaged: 2025-07-25 15:18:53 UTC; karlines
Author: Karline Soetaert [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4603-7100>)
Maintainer: Karline Soetaert <karline.soetaert@nioz.nl>
Repository: CRAN
Date/Publication: 2025-07-25 16:30:02 UTC
